Package: recolorize
Title: Color-Based Image Segmentation
Version: 0.2.0
Authors@R: 
    person(given = "Hannah",
           family = "Weller",
           role = c("aut", "cre"),
           email = "hannahiweller@gmail.com",
           comment = c(ORCID = "0000-0002-5252-4282"))
Description: Automatic, semi-automatic, and manual functions for
    generating color maps from images. The idea is to simplify
    the colors of an image according to a metric that is useful for
    the user, using deterministic methods whenever possible.
    Many images will be clustered well using the out-of-the-box
    functions, but the package also includes a toolbox of functions
    for making manual adjustments (layer merging/isolation, blurring,
    fitting to provided color clusters or those from another image, etc).
    Also includes export methods for other color/pattern analysis packages
    (pavo, patternize, colordistance).
License: CC BY 4.0
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.1
Imports: imager, stats, png, pavo, grDevices, graphics, mgcv,
        colorRamps, plotfunctions, abind, raster, plot3D
Depends: R (>= 4.1.0)
Suggests: knitr, rmarkdown, sf, smoothr, clue, spatstat.geom, methods
VignetteBuilder: knitr
URL: https://hiweller.github.io/recolorize/,
        https://github.com/hiweller/recolorize
BugReports: https://github.com/hiweller/recolorize/issues
NeedsCompilation: no
Packaged: 2025-03-03 19:39:20 UTC; hannahwe
Author: Hannah Weller [aut, cre] (<https://orcid.org/0000-0002-5252-4282>)
Maintainer: Hannah Weller <hannahiweller@gmail.com>
Repository: CRAN
Date/Publication: 2025-03-03 21:00:03 UTC
Built: R 4.6.0; ; 2026-04-29 00:35:58 UTC; unix
