# Generated by roxygen2: do not edit by hand

S3method(plot,recolorize)
S3method(plot,recolorizeVector)
export(absorbLayer)
export(add_image)
export(adjust_color)
export(assignPixels)
export(backgroundCondition)
export(backgroundIndex)
export(blurImage)
export(brick_to_array)
export(classify_recolorize)
export(colorClusters)
export(colorResiduals)
export(constructImage)
export(editLayer)
export(editLayers)
export(hclust_color)
export(imDist)
export(imHeatmap)
export(imposeColors)
export(match_colors)
export(medianColors)
export(mergeLayers)
export(plotColorClusters)
export(plotColorPalette)
export(plotImageArray)
export(raster_to_array)
export(readImage)
export(recluster)
export(recoloredImage)
export(recolorize)
export(recolorize2)
export(recolorizeVector)
export(recolorize_adjacency)
export(recolorize_to_patternize)
export(recolorize_to_png)
export(reorder_colors)
export(rerun_recolorize)
export(splitByColor)
export(thresholdRecolor)
export(wernerColor)
